### Multinomial Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("C:/Users/Administrator/desktop/ppdata.csv",header=TRUE)
## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(10)

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  ## Travel time to destination
  b1 <- x[1]
  ## Travel cost
  b2 <- x[2]
  ## Gender
  b3 <- x[3]
  ## Purpose
  b4 <- x[4]
  ## Out of vehicle time
  b5 <- x[5]
  ## Alternative specific constants
  b6 <- x[6]
  b7 <- x[7]
  b8 <- x[8]
  b9 <- x[9]
  b10 <- x[10]
  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 5 modes
  ## rail
  ## bus 
  ## car
  ## walk
  ## bicycle
  
  ## calculate the utility function
  
  car <- Data$ModeAvailableCar * exp(b1 * Data$TimeCar/10 + b2 * Data$DistanceCar * 13 / 1000)
  bus <- Data$ModeAvailableBus * exp(b10 * Data$TimeBus/10 + b2 * Data$FareBus + b5 * Data$OutofvehicleTimeBus/10 + b9*matrix(1,nrow =hh,ncol=1))
  rail <- Data$ModeAvailableTrain * exp(b10 * Data$TimeTrain/10 + b2 * Data$FareTrain + b3 * Data$Male + b4 * Data$PurposeENG  + b5 * Data$OutofvehicleTimeTrain/10 + b6*matrix(1,nrow =hh,ncol=1))
  walk <- Data$ModeAvailableWalk * exp(b1 * Data$TimeWalk/10 + b8*matrix(1,nrow =hh,ncol=1))
  bicycle <- Data$ModeAvailableBike * exp(b1 * Data$TimeBike/10 + b7*matrix(1,nrow =hh,ncol=1))
  
  ### Calculate the choice probabilities
  deno <- (car + rail + bus + walk + bicycle)
  
  ## Calculate indiviudal choice probabilitiesOutofvehicleTimeTrain
  Prail <- Data$ModeAvailableTrain * (rail / deno)
  Pbus  <- Data$ModeAvailableBus * (bus / deno)
  Pcar  <- Data$ModeAvailableCar * (car / deno)
  Pwalk <- Data$ModeAvailableWalk * (walk / deno)
  Pbicycle <- Data$ModeAvailableBike * (bicycle / deno)
  
  ## Avoid problems stemming from choice probabilities becoming zero.
  Prail <- (Prail!=0) * Prail + (Prail == 0)
  Pbus  <- (Pbus !=0) * Pbus   + (Pbus == 0)
  Pcar  <- (Pcar !=0) * Pcar   + (Pcar == 0)
  Pwalk  <- (Pwalk !=0) * Pwalk   + (Pwalk == 0)
  Pbicycle  <- (Pbicycle !=0) * Pbicycle  + (Pbicycle == 0)
  
  ## Choice results
  Crail   <- Data$MainModeENG == "Rail"
  Cbus    <- Data$MainModeENG == "Bus"
  Ccar    <- Data$MainModeENG == "Car"
  Cwalk    <- Data$MainModeENG == "Walk"
  Cbicycle    <- Data$MainModeENG == "Bicycle"
  
  ## Calculate the Log-likelihood function
  LL <- colSums(Crail*log(Prail) + Cbus*log(Pbus) + Ccar *log(Pcar) 
                + Cwalk *log(Pwalk) + Cbicycle *log(Pbicycle))
}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
