### Nested Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("C:/Users/Administrator/desktop/ppdata.csv",header=T)
## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(11) 

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  
  ## Travel time for priviate
  b1 <- x[1]
  
  ## Travel cost
  b2 <- x[2]
  
  ## Gender
  b3 <- x[3]
  
  ## Purpose
  b4 <- x[4]
  
  ## Out of vehicle time
  b5 <- x[5]
  
  ## Alternative specific constants
  b6 <- x[6]
  b7 <- x[7]
  b8 <- x[8]
  b9 <- x[9]
  
  ## travel time on vehicle
  
  b10 <- x[10]
  
  ##Scale parameter
  pa <- x[11] 

  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
                                        
  train  <- Data$ModeAvailableTrain * exp(b10 * Data$TimeTrain/10+ b2 * Data$FareTrain + b3 * Data$Male + b4 * Data$PurposeENG  + b5 * Data$OutofvehicleTimeTrain/10 + b6*matrix(1,nrow =hh,ncol=1))
  bus    <- Data$ModeAvailableBus * exp(b10 * Data$TimeBus/10 + b2 * Data$FareBus + b5 * Data$OutofvehicleTimeBus/10 + b9*matrix(1,nrow =hh,ncol=1))
  car    <- Data$ModeAvailableCar * exp(b1 * Data$TimeCar/10 + b2 * Data$DistanceCar * 13 / 1000)
  bike   <- Data$ModeAvailableBike * exp(b1 * Data$TimeBike/10 + b7*matrix(1,nrow =hh,ncol=1))
  walk   <- Data$ModeAvailableWalk * exp(b1 * Data$TimeWalk/10 + b8*matrix(1,nrow =hh,ncol=1))

  ### Construct nests.First, calculate public or private choice probabilityP(LV1)
  ## setting of log-sum variables
  logsum.public  <- log( ( (train+bus)!=0) * (train + bus) + ((train+bus)==0))
  logsum.private <- log( ( (car + bike + walk)!=0) * (car + bike + walk) + ((car + bike + walk)==0))

  ##Calculate public or private choice probability with these log-sum variables
  nume.public   <- (logsum.public != 0)*exp(pa*logsum.public) + (logsum.public == 0)
  nume.private  <- (logsum.private != 0)*exp(pa*logsum.private) + (logsum.private == 0)

  deno <- nume.public + nume.private
  P.public  <- nume.public  / deno
  P.private <- nume.private / deno

  ### Next, calculate conditional probability.P(LV2|LV1)
  ## conditional probability under public choice
  deno.public     <- train + bus
  P.train.public  <- train / ((deno.public!=0)*deno.public + (deno.public==0))
  P.bus.public    <- bus   / ((deno.public!=0)*deno.public + (deno.public==0))

  ## conditional probability under private choice
  deno.private    <- car + bike + walk
  P.car.private   <- car   / ((deno.private!=0)*deno.private + (deno.private==0))
  P.bike.private  <- bike  / ((deno.private!=0)*deno.private + (deno.private==0))
  P.walk.private  <- walk  / ((deno.private!=0)*deno.private + (deno.private==0))

  ### Finally, calculate joint probability. P(LV1, LV2) = P(LV2|LV1)*P(LV1)
  P.train <- P.train.public * P.public
  P.bus   <- P.bus.public   * P.public
  P.car   <- P.car.private  * P.private
  P.bike  <- P.bike.private * P.private
  P.walk  <- P.walk.private * P.private

  ## Avoid problems stemming from choice probabilities becoming zero.
  P.train <- (P.train!=0)*P.train + (P.train==0)
  P.bus   <- (P.bus!=0)  *P.bus   + (P.bus==0)
  P.car   <- (P.car!=0)  *P.car   + (P.car==0)
  P.bike  <- (P.bike!=0) *P.bike  + (P.bike==0)
  P.walk  <- (P.walk!=0) *P.walk  + (P.walk==0)

  ## Choice results
  C.train   <- Data$MainModeENG =="Rail"
  C.bus     <- Data$MainModeENG =="Bus"
  C.car     <- Data$MainModeENG =="Car"
  C.bike    <- Data$MainModeENG =="Bicycle"
  C.walk    <- Data$MainModeENG =="Walk"

  ## Calculate the Log-likelihood function
  LL <- colSums(C.train*log(P.train) + C.bus*log(P.bus) + C.car  *log(P.car)   + C.bike  *log(P.bike) +C.walk *log(P.walk))

}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation、Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
